<?php
// +----------------------------------------------------------------------+
// | papaan                                                               |
// +----------------------------------------------------------------------+
// | Copyright (c) 2001,2002,2003 Bitflux GmbH                            |
// +----------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU General Public License (GPL)    |
// | as published by the Free Software Foundation; either version 2       |
// | of the License, or (at your option) any later version.               |
// | The GPL can be found at http://www.gnu.org/licenses/gpl.html         |
// +----------------------------------------------------------------------+
// | Author: Christian Stocker <chregu@bitflux.ch>                        |
// +----------------------------------------------------------------------+
//
// $Id: sitemap.php,v 1.20 2003/07/23 17:35:39 glandium Exp $

/**
* Class for doing the sitemap parsing stuff
*
* @author   Christian Stocker <chregu@bitflux.ch>
* @version  $Id: sitemap.php,v 1.20 2003/07/23 17:35:39 glandium Exp $
* @package  papaan2
*/

include_once(PAPAAN_BASE.'/components/request.php');

class sitemap {


	var $maps = array();
	var $file = null;
	var $uri = '';
   	var $xml = null;
	var $base_dir = '';
	var $parent = NULL; /* NULL if root sitemap */
	var $components = array();
	var $default_components = array();
	var $internal = false; /* false if not internal request */
	var $serialize = true; /* true if we want the sitemap to serialize */
	var $db_pools = array(); /* database connection pools */
	var $view = null;
	var $request = array();

    /**
     * Contains the header, which will be output just before serializing
     *
     * @var array
     */
    var $header = array();

    /**
     * The contenttype
     *
     * should be set within components (resp. serializers) with
     * $this->sitemap->setContentType($type);
     *
     * @var string contenttype
     */

    var $contentType = 'text/html';
	/**
     * The Directory, where the cached sitemaps should be saved
     *
     * @var string
     */
  	var $cacheDir = './tmp/';


    /**
     * Contains instance of ComponentCache or false
     *
     * @var mixed
     * @see enableCaching(), disableCaching()
     */
    var $componentCache = false;

    var $outputCache = false;
    /**
    * The directory, where the xslt documents for
    * sitemap 2 php translation are.
    *
    * @var string
    */
    var $sm2php_xsl_dir;

	/**
     * The XSL File for transforming sitemap.xsl to a cached php file
     *
     * You shouldn't have to change it.
     *
     * @var string
     */
	var $sm2php_xsl = 'sitemap2php.xsl';

	/**
     * Construtor
     *
     *  Almost everything happens here. The cached sitemap is generated here, if it
     *   doesn't exist or if it's older
     *  Then this sitemap is included and the code in it is run
     *
     *  @param string $sitemapFile the location of sitemap.xml, can be relativ
     *  @param string $uri the uri of the call, optional (takes _SERVER['REQUEST_URI'] then)
     *  @param array $options options, to be defined
     *  @access public
     *  @return bool
     */
	function sitemap($sitemapFile, $uri=null, $options = array(), $parent = NULL) {

	if ($parent) {
		$this->parent = &$parent;
		$this->components = $parent->components;
		$this->default_components = $parent->default_components;
		$this->db_pools = $parent->db_pools;
		$this->view = $parent->view;
		$this->request = &$parent->request;
	}
	$this->root = &$this;
	while ($this->root->parent) {
		$this->root = &$this->root->parent;
	}
	$this->sm2php_xsl_dir = PAPAAN_BASE.'/sitemap/';
		//replace class-properties by values in the options-array()
        if (!isset($options['sm2php_xsl_dir']) && isset($options['sm2php_xsl'])) {
            $options['sm2php_xsl_dir'] = dirname($options['sm2php_xsl']);
            $options['sm2php_xsl'] = basename($options['sm2php_xsl']);
        }

		foreach($options as $key => $value) {
			if (isset($this->$key)) {
				$this->$key = $value;
			}
		}
		$this->view = $this->request->get_parameter('papaan-view');
		$this->file = $sitemapFile;

		$this->uri = $uri;

		//generate paths and ids
        $sitemapRealPath = realpath($sitemapFile);
		if (!$sitemapRealPath) {
			return papaan::raiseError('Sitemap '.$sitemapFile.' does not exist',PAPAAN_ERROR_FATAL);
		}
		$this->sitemapId = $this->generateSitemapID($sitemapRealPath);

		$this->sitemapCachedFile = $this->cacheDir . $this->sitemapId;
		//check if sitemapCache does exists and if it's older than the sitemap.xml
        if (! (file_exists($this->sitemapCachedFile) && (filemtime($this->sitemapCachedFile) >= filemtime($sitemapRealPath)) && (filemtime($this->sitemapCachedFile) >= filemtime($this->sm2php_xsl_dir.'/'.$this->sm2php_xsl))))
            {
                //if it is, make new sitemapCached file
                $err = $this->sitemap2php($sitemapRealPath,$this->sitemapCachedFile);
            }

	$this->base_dir = dirname($sitemapFile);
	}

	/**
     * Runs the cached Sitemap
     *
     * @param string $sitemapCachedFile location of the cached file
     * @return mixed true on success, pear error on error
     */
	function runSitemap() {
          include_once($this->sitemapCachedFile);
	  return call_user_func($this->sitemapId,&$this);
	}

    /**
     * Sets a header, which is output just before the serializer
     *
     * @param string $name name of the header
     * @param string $value value of the header
     */
    function setHeader($name, $value) {
        $this->header[$name] = $value;
    }

    /**
     * Sets the last modified time
     *
     * @param int $time unixtime last modified
     */
    function setLastModified($time) {
        if ($time > 0) {
            $this->setHeader('Last-Modified',gmdate('r', $time));
        }
    }

    /**
     * Sets the content type of the document
     *
     * @param string $type content type
     */
    function setContentType($type) {
        $this->contentType = $type;
    }

    /**
    * Prints all the header in $this->header
    *
    * this function is called from components/serializer.php in the constructor
    */

    function printHeader() {
        $error[404] = '404 Not Found';
	$error[500] = '500 Internal Server Error';
        $this->setHeader('Content-Type',$this->contentType);
        foreach ($this->header as $name => $value) {
	    if (strtolower($name) == 'status') {
		$value = $error[$value];
            	header('HTTP/1.0 '.$value);
	    }
            header("$name: $value");
        }
    }


	/**
     * generates a unique ID out of a string (path+filename)
     *
     * This method is used for generating a unique ID for every sitemap
     *  which has to be compiled. It just takes the realpath as input,
     *  and returns a unique ID for it.
     * In this case it replaces every DIRECTORY_SEPERATOR with _ for better
     *  debugging. Theoretically one could use md5 as well.
     *
     * @param string $realpath Any string, but in this class normally a fullpath+filename
     * @access public
     * @return string ID
     */
	function generateSitemapID ($realpath) {
		return str_replace(array(':','-','.','+','\\','/'),'_',$realpath);
	}

	/**
     * Generates the cached sitemap
     *
     * This function generates a php file out of a sitemap.xml with the help
     *  an xsl file (poponn/sitemap/sitemap2xsl.php)
     *
     * Libxslt and Sablotron is supported right now.
     *
     * @param string $sitemapRealPath the absolute location of sitemap.xml
     * @param string $sitemapCachedFile the absolute location of the cached sitemap (php file)
     * @access public
     * @return bool
     */
	function sitemap2php($sitemapRealPath, $sitemapCachedFile) {
		//file_exists and is writable should be the normal case...
		if (! is_writable($sitemapCachedFile))
            {
                if (! realpath(dirname($sitemapCachedFile)))
                    {
                        if (!mkdir(dirname($sitemapCachedFile).'/')) {
                            return papaan::raiseError('The cache directory '. dirname($sitemapCachedFile).' does not exist' ,PAPAAN_ERROR_FATAL);
                        }
                    }
                if (!is_writable(dirname($sitemapCachedFile)))
                    {
                        return papaan::raiseError('The cache directory  '. realpath(dirname($sitemapCachedFile)).' is not writable',PAPAAN_ERROR_FATAL,__FILE__,__LINE__);
                    }
                else if ((file_exists($sitemapCachedFile) && !is_writable($sitemapCachedFile)))
                    {
                        return papaan::raiseError('The cache file '.realpath($sitemapCachedFile). ' is not writable',PAPAAN_ERROR_FATAL);
                    }
            }

                $xsl = domxml_xslt_stylesheet_file($this->sm2php_xsl_dir.'/'.$this->sm2php_xsl);
                $sm = @domxml_open_file($sitemapRealPath,0,$errors);
                if ($errors) {
                    print '<pre>';
                    print_r ($errors);
                }
                $result = $xsl->process($sm,array('ID'=>$this->sitemapId));

                // check if we have result_dump_file()
                // result_dump_file could be faster than the other function
                //  and respects omit-xml-declaration
                if (method_exists($xsl,'result_dump_file'))
                    {
                        if (! $xsl->result_dump_file($result,$sitemapCachedFile))
                            {
                                return papaan::raiseError('Could not save '.$sitemapCachedFile.'. (In '. __FILE__ .':'.__LINE__.')',PAPAAN_ERROR_FATAL);
                            }
                    }
                // else use traditional dump_mem
                else
                    {
                        //PERFORMANCE: always turn off short_open_tag, otherwise we have to strip it from the sm-xsl2php output
                        $shortOpen = ini_get('short_open_tag');
                        $shortOpen = (int) $shortOpen || $shortOpen == 'On';

                        // if shortTags are open, we have to strip away the xml processing instruction
                        //  otherwise php chokes on it.
                        if ($shortOpen)
                            {
                                $result = $result->dump_mem();
                                $result = papaan::stripXmlPI($result);
                                papaan::saveFile($result,$sitemapCachedFile);
                            }
                        else
                            {
                                $result->dump_file($sitemapCachedFile);
                            }
                    }

		return True;
	}

    /**
     * Converts it's parameter into a DomDocument object
     *
     * @param  mixed  xmldoc        can either be a XML String or a DomDocument object
     * @return bool
     * @access private
     */
    function var2XMLObject(&$xmldoc)
    {
        if (is_string ($xmldoc))
        {

            // we have 4.3, so we can use the 'nicer' errorsystem
            $_xmldoc = @xmldoc($xmldoc, 0,&$errors);
            if ($errors) {
                print '<pre>';
                print 'XML ERRORS!<br/>';
                print_r($errors);
                print '<hr>';
                print 'The XML document was:';
                print '<hr>';
                print '<pre>';
                $xmldoc = explode ("\n",$xmldoc);
                $_len = count($xmldoc);
                foreach($errors as $error) {
                    $lineerrors[$error['line']] = $error['col'];
                }
                for ($i=0; $i < $_len; $i++) {
                    print htmlentities($xmldoc[$i]);
                    print "\n";
                    if (isset($lineerrors[$i + 1])) {
                        print '<font color="red">';
                        print str_repeat('-',$lineerrors[$i + 1] -1);
                        print '^';
                        print '</font>'."\n";
                    }
                }
            }
            $xmldoc = $_xmldoc;
            unset ($_xmldoc);
        }
        if ( strtolower(get_class($xmldoc)) != 'domdocument')
            {
                return papaan::raiseError('First parameter to var2XMLObject() is neither a XML String nor a XML DomDocument object. It is: '.
                                          var_export($xmldoc, true),
                                          PAPAAN_ERROR_FATAL);
            }
        return True;
    }

    function redirectTo($uri) {
        header('Location: '.$uri);
        exit;
    }

    function translateScheme($value) {
/*	if (substr ($value, -1,1) == '}' && strrpos ($value, '{') === 0) {
		$value = substr($value, 1, -1);
		$value = @$this->maps[substr_count($value,'../')][str_replace('../','',$value)];
	} else {*/
                $value = preg_replace('#\{([\./]*([^}]+))\}#e','@$this->maps[substr_count(\'$1\',\'../\')][\'$2\']',$value );
//	}
    	return $value;
    }

	function addMap($map) {
		array_unshift($this->maps,$map);
	}

	function removeMap() {
        array_shift($this->maps);
	}

    function setGlobalOptions($name, $data) {
        $GLOBALS['_PAPAAN_globalContainer']->options[$name] = $data;
    }

    function setGlobalOptionsAll( $data) {
        $GLOBALS['_PAPAAN_globalContainer']->options = $data;
    }

    function getGlobalOptions($name) {
        return $GLOBALS['_PAPAAN_globalContainer']->options[$name] ;
    }

    function getGlobalOptionsAll() {
        if (isset($GLOBALS['_PAPAAN_globalContainer']->options)) {
            return $GLOBALS['_PAPAAN_globalContainer']->options ;
        }
        else
        {
            return null;
        }
    }


	/**
     * Mounts a second sitemap
     *
     * Not a very elegant solution, should be rewritten some day
     *
     */
    function _mount($attribs)
    {
		$file = $attribs['src'];
		$uri = $this->uri;
		if (isset($attribs['uri-prefix']))
            {
                $prefix = $attribs['uri-prefix'];
                if ($prefix)
                    {
                        $uri = preg_replace('#^'.$prefix.'/*#','',$uri);
                    }

            }
		// I hope, this doesn't have too many sideeffects
		// Would need to pass options to child sitemap object
		$sitemap = new sitemap($this->base_dir.'/'.$file,$uri,array('internal'=>$this->internal),$this);
		$status = $sitemap->runSitemap();
		$this->xml = $sitemap->xml;
		$this->header = $sitemap->header;
		$this->contentType = $sitemap->contentType;
		return $status;
    }

    function output() {
    	// print headers only if internal request
	if (!$this->internal) {
		$this->printHeader();
	}
	if (is_string($this->xml)) {
		print $this->xml;
	} else {
		papaan::raiseError('Output should be a string ; something went wrong...',PAPAAN_ERROR_FATAL);
	}
    }

    /** Creates a new component
     */
    function new_component($type, $name) {
	if ($type) {
		if ($name == '') {
			$name = $this->default_components[$type.'s'];
		}
    		if (isset($this->components[$type][$name])) {
			if (isset($this->components[$type][$name]['dom']) && is_string($this->components[$type][$name]['dom'])) {
				$this->components[$type][$name]['dom'] = xmldoc($this->components[$type][$name]['dom']);
			}
			$class = $this->components[$type][$name]['src'];
			$phpfile = PAPAAN_BASE.'/components/'.$type.'s/'.$class.'.php';
			if (is_file($phpfile)) {
				include_once(PAPAAN_BASE.'/components/'.$type.'s/'.$class.'.php');
				if (class_exists($type.'_'.$class)) {
					return(eval('return new '.$type.'_'.$class.'($this,$this->components[$type][$name]);'));
				}
			}
		}
	}

	papaan::raiseError("Undefined $type component: $name",PAPAAN_ERROR_FATAL);
	
    }

    function check_view($label, $type=null, $name=null) {
	if (!isset($this->views[$this->view])) {
		return false;
	}
	$view = $this->views[$this->view];
	if ($view ===1) {
		return true;
	}
	if ($type == 'serializer') {
		if ($view === -1) {
			return true;
		}
		return false;
	}
	$label = split(',',$label);
	if ($label and in_array($view, $label)) {
		return true;
	}
    	if ($type) {
		if ($name == '') {
			$name = $this->default_components[$type.'s'];
		}
		if (isset($this->components[$type][$name]) &&
		    isset($this->components[$type][$name]['label']) &&
		    is_array($this->components[$type][$name]['label'])) {
		    	return in_array($view,$this->components[$type][$name]['label']);
		}
	}
	return false;
    }

    /** Raises an error
     */

    function raiseError($component, $type, $message, $description = null) {

	if (is_object($component)) {
		$comp = get_class($component);
	} else {
		$comp = $component;
	}
	if (! $description) {
		$description = $message;
	}
	$sitemap = $this;
	if ($type != 'fatal') {
		while(!(isset($sitemap->errorHandler) && $sitemap->errorHandler) && $sitemap->parent) {
			$sitemap = $sitemap->parent;
		}
		if (isset($sitemap->errorHandler) && $sitemap->errorHandler) {
			$errorHandler = $sitemap->errorHandler;
			$sitemap->errorHandler = null;
			$sitemap->error = array('component' => $comp, 'type' => $type, 'message' => $message, 'description' => $description);
			call_user_func($errorHandler, &$sitemap);
			$sitemap->output();
			exit;
		}
	}
	papaan::Error($comp, $type, $message, $description);
    }

    /**
     * Starts component caching
     *
     * @param  array  Attributes of current pipeline
     * @see    disableCaching(), $componentCache
     */
    function enableCaching($pipelineAttribs){
        include(PAPAAN_BASE . '/components/cache.php');
        $this->componentCache = new ComponentCache($pipelineAttribs, $this);
    }

    /**
     * Disables component caching
     *
     * Reset $this->componentCache
     *
     * @see    enableCaching(), $componentCache
     */
    function disableCaching(){
        $this->componentCache = false;
    }

    function disableOutputCaching() {
       $this->outputCache = false;
    }

}

?>
