<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:map="http://apache.org/cocoon/sitemap/1.0"
>
    <xsl:output omit-xml-declaration="yes" indent="no" encoding="ISO-8859-1" method="xml"/>
    <xsl:include href="../common.xsl"/>
    <xsl:param name="ID"/>

    <xsl:template match="/map:sitemap"><xsl:processing-instruction name="php">
	    $sitemap = &amp;$this;
            <xsl:apply-templates select="map:components"/>
            <xsl:apply-templates select="map:resources"/>
            <xsl:apply-templates select="map:views"/>
            <xsl:apply-templates select="map:action-sets"/>
            <xsl:apply-templates select="map:pipelines"/>
            <xsl:apply-templates select="map:pipelines/map:pipeline/map:handle-errors"/>
    </xsl:processing-instruction></xsl:template>
    
    <xsl:template name="tag">
	<xsl:text>&lt;</xsl:text>
	<xsl:value-of select="name()"/>
	<xsl:for-each select="@*">
		<xsl:text> </xsl:text>
		<xsl:value-of select="name()"/>
		<xsl:text>="</xsl:text>
		<xsl:call-template name="escapeSingleQuotes">
		<xsl:with-param name="text" select="normalize-space(.)"/>
		</xsl:call-template>
		<xsl:text>"</xsl:text>
	</xsl:for-each>
	<xsl:if test="namespace-uri()">
		<xsl:text> xmlns</xsl:text>
		<xsl:if test="substring-before(name(),':')">
			<xsl:text>:</xsl:text>
			<xsl:value-of select="substring-before(name(),':')"/>
		</xsl:if>
		<xsl:text>="</xsl:text>
		<xsl:value-of select="namespace-uri()"/>
		<xsl:text>"</xsl:text>
	</xsl:if>
	<xsl:choose>
		<xsl:when test="*|text()">
			<xsl:text>&gt;</xsl:text>
			<xsl:apply-templates select="*|text()"/>
			<xsl:text>&lt;/</xsl:text>
			<xsl:value-of select="name()"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>/</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>&gt;</xsl:text>
    </xsl:template>

    <xsl:template match="map:matchers/map:matcher|map:serializers/map:serializer|map:generators/map:generator|map:readers/map:reader|map:transformers/map:transformer|map:selectors/map:selector|map:actions/map:action">
	$this->components['<xsl:value-of select="local-name()"/>']['<xsl:value-of select="@name"/>'] = array(
	<xsl:if test="@label">'label'=>array(<xsl:call-template name="replace-string"><xsl:with-param name="text"><xsl:call-template name="translateString"><xsl:with-param name="text" select="@label"/></xsl:call-template></xsl:with-param><xsl:with-param name="search">,</xsl:with-param><xsl:with-param name="replace">','</xsl:with-param></xsl:call-template>),</xsl:if>
        <xsl:for-each select="@*[name()!='name' and name()!='label']">'<xsl:value-of select="name()"/>'=><xsl:call-template name="translateString"><xsl:with-param name="text" select="."/></xsl:call-template>,</xsl:for-each>
	<xsl:if test="*">
	'dom' =>
	'&lt;config&gt;<xsl:apply-templates select="*"/>&lt;/config&gt;'
	</xsl:if>);
    </xsl:template>

    <xsl:template match="map:matcher//text()|map:serializer//text()|map:generator//text()|map:reader//text()|map:transformer//text()|map:selector//text()|map:action//text()">
	<xsl:call-template name="escapeSingleQuotes">
	<xsl:with-param name="text" select="normalize-space(.)"/>
	</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:matcher//*|map:serializer//*|map:generator//*|map:reader//*|map:transformer//*|map:selector//*|map:action//*">
    	<xsl:call-template name="tag"/>
    </xsl:template>

    <xsl:template match="map:components/map:matchers|map:components/map:serializers|map:components/map:generators|map:components/map:transformers|map:components/map:readers|map:components/map:selectors|map:components/map:actions">
	$this->default_components['<xsl:value-of select="local-name()"/>'] = '<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="@default"/></xsl:call-template>';
        <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="map:sitemap/map:pipelines/map:pipeline/map:handle-errors">
	function <xsl:value-of select="concat($ID,'_',generate-id())"/>(&amp;$sitemap) {
        <xsl:apply-templates />
	}
    </xsl:template>

    <xsl:template match="map:sitemap/map:pipelines">
	function <xsl:value-of select="$ID"/>(&amp;$sitemap) {
		    $pipelineHit = false;
        <xsl:apply-templates />
		    return false;
	}
    </xsl:template>

    <xsl:template match="map:views/map:view">
	$this->views[<xsl:call-template name="translateString"><xsl:with-param name="text" select="@name"/></xsl:call-template>] = <xsl:choose>
	<xsl:when test="@from-position='last'">-1</xsl:when>
	<xsl:when test="@from-position='first'">1</xsl:when>
	<xsl:otherwise><xsl:call-template name="translateString"><xsl:with-param name="text" select="@from-label"/></xsl:call-template></xsl:otherwise>
	</xsl:choose>;

	<xsl:call-template name="views"/>
    </xsl:template>

    <xsl:template match="map:action-sets/map:action-set">
	function action_set_<xsl:value-of select="$ID"/>_<xsl:value-of select="translate(@name,'-+','__')"/>(&amp;$sitemap) {
		$result = array();
		<xsl:apply-templates/>
		return $result;
    	}
    </xsl:template>
    <xsl:template match="map:resources/map:resource" name="views">
	function <xsl:value-of select="local-name()"/>_<xsl:value-of select="$ID"/>_<xsl:value-of select="translate(@name,'-+','__')"/>(&amp;$sitemap) {
		    $pipelineHit = false;
        <xsl:apply-templates />
		    return $pipelineHit;
	}
    </xsl:template>

    <xsl:template match="map:sitemap/map:components|map:sitemap/map:resources|map:sitemap/map:views|map:sitemap/map:action-sets">
        <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="map:pipelines/map:pipeline/map:parameter[@name = 'expires']">    
        <xsl:if test="starts-with(@value,'access plus')">
            $sitemap->setHeader('Expires',gmdate('r',strtotime('<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="substring-after(@value,'access plus')"/></xsl:call-template>')));
        </xsl:if>

    </xsl:template>
    
    <xsl:template match="map:pipelines/map:pipeline">
        
	$sitemap->errorHandler = null;
	<xsl:if test="@internal-only='true'">
	if ($sitemap->internal) {
	</xsl:if>

	<xsl:choose>
	<xsl:when test="count(map:handle-errors) &gt; 1">
        $sitemap->raiseError('sitemap','fatal','<xsl:value-of select="name(map:handle-errors)"/> is not allowed more than once in a <xsl:value-of select="name()"/>.');
	</xsl:when>
	<xsl:when test="map:handle-errors/@type">
        $sitemap->raiseError('sitemap','fatal','Attribute type is deprecated for element <xsl:value-of select="name(map:handle-errors)"/>.');
	</xsl:when>
	<xsl:when test="count(map:handle-errors) = 1">
	$sitemap->errorHandler = '<xsl:value-of select="concat($ID,'_',generate-id(map:handle-errors))"/>';
	</xsl:when>
	</xsl:choose>

        <xsl:if test="@benchmark">
            $sitemap->benchmark['do'] = True;
        </xsl:if>
    
        <xsl:choose>
	   <xsl:when test="@cachable = 'yes' or @cachable = 'true'">
              $sitemap->enableCaching(<xsl:call-template name="generateAttributes"/>);
           </xsl:when>
           <xsl:otherwise>
              $sitemap->disableCaching();
           </xsl:otherwise>
        </xsl:choose>
        
        <xsl:apply-templates select="*[not(local-name() = 'handle-errors' and namespace-uri() = namespace-uri(..))]"/>

	if ($pipelineHit) {
		return true;
	<xsl:if test="not(following-sibling::*[local-name() = 'pipeline' and namespace-uri() = namespace-uri(..) and not(@internal-only)])">
	} else {
		$sitemap->raiseError('sitemap','resource-not-found','Resource not found','The requested URI "/'.$sitemap->root->uri.'" was not found.');
	</xsl:if>
	}

	<xsl:if test="@internal-only='true'">
	}
	</xsl:if>
        
    </xsl:template>

	<xsl:template match="map:pipeline//map:mount|map:resource//map:mount">
		return ($sitemap->_mount(<xsl:call-template name="generateAttributes"/>));
	</xsl:template>		

	<xsl:template match="map:pipeline//map:aggregate|map:resource//map:aggregate">
	   	include_once(PAPAAN_BASE.'/components/aggregator.php');
        $aggregator = &amp;new aggregator($sitemap);

		$aggregator->init(<xsl:call-template name="generateAttributes"/>);
	<xsl:if test="map:part[@label]">
	$has_view = false;
	<xsl:for-each select="map:part[@label]">
	if ($sitemap->check_view(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@label"/></xsl:call-template>)) {
		$has_view = true;
		<xsl:apply-templates select="."/>
	}
	</xsl:for-each>
	if ($has_view) {
		$aggregator->start($sitemap->xml);
		return call_user_func('view_<xsl:value-of select="$ID"/>_'.$sitemap->view,&amp;$sitemap) || $pipelineHit;
	}
	</xsl:if>
		<xsl:apply-templates/>
		$aggregator->start($sitemap->xml);
	<xsl:call-template name="use-view"/>
	</xsl:template>		
	
	<xsl:template match="map:aggregate/map:part">
		$aggregator->addPart(<xsl:call-template name="generateAttributes"/>);
	</xsl:template>		

    <xsl:template match="map:pipeline//map:generate|map:resource//map:generate" name="doGenerate">
        <xsl:param name="xmlVar" select="'$sitemap->xml'"/>
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">generator</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>						
        </xsl:call-template>

                if($sitemap->componentCache AND $sitemap->componentCache->init($generator)){
                                                            
                    if(!$sitemap->componentCache->isCached()){
                        /* miss! :( */
                        $generator->DomStart(<xsl:value-of select="$xmlVar"/>);          
                        $sitemap->componentCache->store(<xsl:value-of select="$xmlVar"/>);
                    }
                    else{
                        /* hit! :)  */
                    }
                }
                else $generator->DomStart(<xsl:value-of select="$xmlVar"/>);
            
                $pipelineHit = True;

	<xsl:call-template name="use-view">
		<xsl:with-param name="prefix">generator</xsl:with-param>
	</xsl:call-template>

    </xsl:template>

    <xsl:template match="map:pipeline//map:read|map:resource//map:read">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">reader</xsl:with-param>
        </xsl:call-template>

        if($sitemap->componentCache AND $sitemap->componentCache->init($reader)){

            if(!$componentCache->isCached()) {
                $sitemap->componentCache->store(null, 'start', null, true);
            }
            else{
                print($componentCache->load());
            }
        
        }
        else{
            $sitemap->xml = $reader->start();
        }
        		
        $pipelineHit = True;
	return true;
        
    </xsl:template>

    
    <xsl:template match="map:pipeline//map:transform|map:resource//map:transform|map:view//map:transform">
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">transformer</xsl:with-param>			
            <xsl:with-param name="doParams">true</xsl:with-param>						
        </xsl:call-template>
        
         if($sitemap->componentCache AND $sitemap->componentCache->init($transformer)){
                if(!$sitemap->componentCache->isCached()){
                        $sitemap->componentCache->loadLast($sitemap->xml);
                        $transformer->DomStart($sitemap->xml);
                        $sitemap->componentCache->store($sitemap->xml);                   
                 }
                   
                else {}
          }
         else{
                $transformer->DomStart($sitemap->xml);
             }
        
	<xsl:if test="not(ancestor::map:view)">
	<xsl:call-template name="use-view">
		<xsl:with-param name="prefix">transformer</xsl:with-param>
	</xsl:call-template>
	</xsl:if>

    </xsl:template>
    
	<xsl:template match="map:act/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">action</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>						
		</xsl:call-template>
    </xsl:template>


    <xsl:template match="map:select/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">selector</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:transform/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">transformer</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:generate/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">generator</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

    <xsl:template match="map:serialize/map:parameter">
		<xsl:call-template  name="setParameter">
			<xsl:with-param name="prefix">serializer</xsl:with-param>
		</xsl:call-template>
    </xsl:template>

	<xsl:template match="map:pipeline//map:act|map:resource//map:act|map:view//map:act|map:action-set/map:act">
	<xsl:choose>
	<xsl:when test="@set">
	    <xsl:variable name="set" select="@set"/>
	    <xsl:choose>
	    <xsl:when test="/map:sitemap/map:action-sets/map:action-set[@name=$set]">
	    if ($map = action_set_<xsl:value-of select="$ID"/>_<xsl:value-of select="translate($set,'+-','__')"/>($sitemap)) {
			$sitemap->addMap($map);
		        <xsl:apply-templates select="*[name() != 'map:parameter']"/>		
			$sitemap->removeMap();
	    }
	    </xsl:when>
	    <xsl:otherwise>
        	$sitemap->raiseError('sitemap','fatal','No <xsl:value-of select="@set"/> action-set defined in your sitemap');
	    </xsl:otherwise>
	    </xsl:choose>
	</xsl:when>
	<xsl:otherwise>
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">action</xsl:with-param>
            <xsl:with-param name="doParams">true</xsl:with-param>
            <xsl:with-param name="doInit">
	    <xsl:choose>
	    <xsl:when test="@src">true</xsl:when>
	    <xsl:otherwise>false</xsl:otherwise>
	    </xsl:choose>
	    </xsl:with-param>
        </xsl:call-template>
	<xsl:apply-templates select="." mode="do"/>
	</xsl:otherwise>
	</xsl:choose>

	</xsl:template>

	<xsl:template match="map:action-sets//map:act" mode="do">
	if (
	<xsl:if test="@action">
	($sitemap->request->get_parameter('papaan-action-<xsl:value-of select="@action"/>') !== null) and
	</xsl:if>
        (($map = $action->act()) !== false)) {
		$result = array_merge($result, $map);
	}
	</xsl:template>

	<xsl:template match="map:pipeline//map:act|map:resource//map:act|map:view//map:act" mode="do">
        if (($map = $action->act()) !== false)
        {
			$sitemap->addMap($map);
		        <xsl:apply-templates select="*[name() != 'map:parameter']"/>		
			$sitemap->removeMap();
        } 

	</xsl:template>
    
    <xsl:template match="map:pipeline//map:serialize|map:resource//map:serialize">
	<xsl:call-template name="use-view">
	<xsl:with-param name="prefix">serializer</xsl:with-param>
	</xsl:call-template>

	<xsl:call-template name="serialize"/>
    </xsl:template>

    <xsl:template match="map:view//map:serialize" name="serialize">
	<xsl:if test="@status-code">
	$sitemap->setHeader('status',<xsl:value-of select="@status-code"/>);
	</xsl:if>
	// only serialize if asked to
	if ($sitemap->serialize) {
        <xsl:call-template name="setupComponent">
            <xsl:with-param name="prefix">serializer</xsl:with-param>			
            <xsl:with-param name="doParams">true</xsl:with-param>						
            <xsl:with-param name="doInit">false</xsl:with-param>						
        </xsl:call-template>

                if($sitemap->componentCache AND $sitemap->componentCache->init($serializer)){
    
                    if($sitemap->componentCache->isCached()){
                        $sitemap->componentCache->load(true);
                    }
                    else{
                        $sitemap->componentCache->loadLast($sitemap->xml);
                        $sitemap->componentCache->store(null, 'DomStart', $sitemap->xml, true);
                        } 
                }

                else{
                   	$sitemap->xml = $serializer->DomStart($sitemap->xml);
                }
	}
	return true;

        
    </xsl:template>
    
    <xsl:template match="map:match">
		<xsl:call-template name="setupComponent">
			<xsl:with-param name="prefix">matcher</xsl:with-param>
            <xsl:with-param name="doInit">false</xsl:with-param>						
		</xsl:call-template>

        if ($matcher->match(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@pattern"/></xsl:call-template>))
        {
        <xsl:apply-templates/>
        } 
        // end of map match = <xsl:value-of select="@pattern"/>
        
        $sitemap->removeMap();

    </xsl:template>
    
    <xsl:template match="map:select">
		<xsl:call-template name="setupComponent">
			<xsl:with-param name="prefix">selector</xsl:with-param>
            		<xsl:with-param name="doParams">true</xsl:with-param>
            <xsl:with-param name="doInit">false</xsl:with-param>						
		</xsl:call-template>

        <xsl:apply-templates select="map:when|map:otherwise"/>
		$selector = null;
    </xsl:template>
    
    <xsl:template match="map:select//map:when">
        
        <xsl:if test="position() > 1">
        else</xsl:if> if ($selector->test(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@test"/></xsl:call-template>)) {

        <xsl:apply-templates />
        } // end when test = <xsl:value-of select="@test"/>
    </xsl:template>
    
    <xsl:template match="map:select//map:otherwise">
        else {
        <xsl:apply-templates />
        } // end else
    </xsl:template>
    

    <xsl:template match="map:pipeline//map:redirect-to|map:resource//map:redirect-to">
        $sitemap->redirectTo(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@uri"/></xsl:call-template>);
	return true;
    </xsl:template>


    <xsl:template match="map:action[@type='disableOutputCaching']">
        $sitemap->disableOutputCaching();
    </xsl:template>
    
    <xsl:template name="generateAttributes">
        array(
        <xsl:for-each select="@*">'<xsl:value-of select="name()"/>'=><xsl:call-template name="translateString"><xsl:with-param name="text" select="."/></xsl:call-template>,</xsl:for-each>
        )
    </xsl:template>
    
    <xsl:template match="*">
        $sitemap->raiseError('sitemap','fatal','<xsl:value-of select="name()"/> is not allowed as a child of <xsl:value-of select="name(..)"/> in your sitemap');
        <xsl:apply-templates/>
    </xsl:template>   
    
    <xsl:template name="setupComponent">
        <xsl:param name="prefix"/>
	<xsl:param name="doParams">false</xsl:param>
	<xsl:param name="doInit">true</xsl:param>

	$<xsl:value-of select="$prefix"/> = &amp;$sitemap->new_component('<xsl:value-of select="$prefix"/>',<xsl:call-template name="translateString"><xsl:with-param name="text" select="@type"/></xsl:call-template>);
		
	<xsl:if test="$doParams != 'false'">
	<xsl:apply-templates select="map:parameter"/>
	</xsl:if>
	<xsl:if test="$doInit != 'false'">
        $<xsl:value-of select="$prefix"/>->init(<xsl:call-template name="generateAttributes"/>);
	</xsl:if>
    </xsl:template>
	
	

	<xsl:template name="setParameter">
		<xsl:param name="prefix"/>
		$<xsl:value-of select="$prefix"/>->setParameter(<xsl:call-template name="translateString"><xsl:with-param name="text" select="@name"/></xsl:call-template>,<xsl:call-template name="translateString"><xsl:with-param name="text" select="@value"/></xsl:call-template>);
		</xsl:template>
    
    <xsl:template name="map">
	<xsl:param name="path"/>
	<xsl:text>$sitemap->maps[0]['</xsl:text>
	<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$path"/></xsl:call-template>
	<xsl:text>']</xsl:text>
    </xsl:template>

    <xsl:template name="translateString">
	<xsl:param name="text"/>
    	<xsl:choose>
		<xsl:when test="$text != ''">
			<xsl:choose>
				<xsl:when test="contains($text,'{') and contains($text,'}')">
					<xsl:text>$sitemap->translateScheme('</xsl:text>
					<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$text"/></xsl:call-template>
					<xsl:text>')</xsl:text>
					<!--
					<xsl:if test="substring-before($text,'{')">
						<xsl:text>'</xsl:text>
						<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="substring-before($text,'{')"/></xsl:call-template>
						<xsl:text>' . </xsl:text>
					</xsl:if>
					<xsl:call-template name="map"><xsl:with-param name="path" select="substring-before(substring-after($text,'{'),'}')"/></xsl:call-template>
					<xsl:if test="substring-after($text,'}')">
						<xsl:text> . </xsl:text>
						<xsl:call-template name="translateString"><xsl:with-param name="text" select="substring-after($text,'}')"/></xsl:call-template>
					</xsl:if>
					-->
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>'</xsl:text>
					<xsl:call-template name="escapeSingleQuotes"><xsl:with-param name="text" select="$text"/></xsl:call-template>
					<xsl:text>'</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
		<xsl:otherwise>
		<xsl:text>''</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
    </xsl:template>

    <xsl:template match="map:pipeline//map:call|map:resource//map:call">
	$tmp = array();
	<xsl:apply-templates select="*"/>
	$sitemap->addMap($tmp);
	return resource_<xsl:value-of select="$ID"/>_<xsl:value-of select="translate(@resource,'+-','__')"/>($sitemap);
	$sitemap->removeMap();
    </xsl:template>

    <xsl:template match="map:call/map:parameter">
	$tmp[<xsl:call-template name="translateString"><xsl:with-param name="text" select="@name"/></xsl:call-template>] = <xsl:call-template name="translateString"><xsl:with-param name="text" select="@value"/></xsl:call-template>;
    </xsl:template>

    <xsl:template name="use-view">
    	<xsl:param name="prefix"/>
	<xsl:param name="label" select="@label"/>

	<xsl:if test="/map:sitemap/map:views/map:view">
	if ($sitemap->view and $sitemap->check_view(<xsl:call-template name="translateString"><xsl:with-param name="text" select="$label"/></xsl:call-template>,<xsl:call-template name="translateString"><xsl:with-param name="text" select="$prefix"/></xsl:call-template>,<xsl:call-template name="translateString"><xsl:with-param name="text" select="@type"/></xsl:call-template>)) {
		return call_user_func('view_<xsl:value-of select="$ID"/>_'.$sitemap->view,&amp;$sitemap) || $pipelineHit;
	}
	</xsl:if>
    </xsl:template>

</xsl:stylesheet>
